'use strict';
/**
 * Created by gongbin on 2017/6/22.
 */
const hessian = require('hessian.js');
var ByteBuffer = require('byte');

const Html5RpcFaultEvent = require("./Html5RpcFaultEvent");
const Html5RpcResultEvent = require("./Html5RpcResultEvent");

const debug = require('debug')('gongbin:Html5RpcOperation');
const ISDEBUG = require("./Html5RpcManager").debug;
const perPackageSize = require("./Html5RpcManager").perPackageSize;
const isChenBinTest = require("./Html5RpcManager").isChenBinTest;

let newDebug = function(msg){
    console.log(msg);
    if(ISDEBUG){
        debug(msg);
    }
}


 class Html5RpcOperation{

    /**
     * 构造函数
     *
     * @param {String} uid
     * @param {String} 包名
     * @param {String} token
     * @param {String} 方法名
     * @param {Array} 参数
     * @param {Function} 成功回调
     * @param {Function} 失败回调
     * @public
     */
    constructor(uid,token,pkgName,methordName,args,socket,successFunc,failureFunc){
        this.uid = uid;
        this.token = token;
        this.pkgName = pkgName;
        this.methordName = methordName;
        this.args = args;
        this.socket = socket;
        this.successFunc = successFunc;
        this.failureFunc = failureFunc;
        this.socket.addOnMessageCallBack(this.onSocketMessage,this);
       // this.packageDataArr = null;
    }

    deleteOperation(){

    }

    onSocketMessage(data){
        if(isChenBinTest){
            // this.doSuccessMessage1(data);
            if (data && data.hasOwnProperty('uuid')) {
                if(data.uuid == this.uid){
                    newDebug("收到数据 uuid为:"+data.uuid+", 信息类型为:"+data.responsType+", 当前的uid为:"+this.uid);

                    if(data.responsType==1){
                        this.doSuccessMessage(data.obj);
                    }
                    else if(data.responsType==3){
                        this.doFailureMessage(data.obj);
                    }
                    else{
                        newDebug("未知的数据包类型:"+data.responsType);
                    }
                    
                }
            }
        }
        else{
            if (data && data.hasOwnProperty('uuid')) {
                if(data.uuid == this.uid){
                    newDebug("收到数据 uuid为:"+data.uuid+", 当前包索引为"+data.packetIndex+", 包数量为:"+data.packetNum+", 当前的uid为:"+this.uid);
                    
                    if(data.packetNum == 1){//没有分包
        
                        if(data.responsType==1){
                            this.doSuccessMessage(data.responseInfoBytes)
                        }
                        else if(data.responsType==3){
                            this.doFailureMessage(data.responseInfoBytes)
                        }
                        else{
                          newDebug("未知的数据包类型:"+data.responsType);
                        }
                    }
                    else{ //分包了
        
                        if(this.packageDataArr == null || this.packageDataArr==undefined){
                            this.packageDataArr = [];
                            this.packageDataToalLength = 0;
                            newDebug("初始化多包数据")
                        }
                        this.packageDataToalLength+= data.responseInfoBytes.length;
                        this.packageDataArr[data.packetIndex-1]=data.responseInfoBytes;
        
                        if(this.packageDataArr.length == data.packetNum){
                            //检查每个元素都不为undefined.
                            var isAllData = true;
                            for(let i = 0 ; i < data.packetNum;i++){
                                if(this.packageDataArr[i] == undefined){
                                    isAllData = false;
                                    break;
                                }
                            }
        
                            if(isAllData == true){
                                newDebug("多包数据接收完成，开始反序列化,总字节数为:"+this.packageDataToalLength);
                                let  newBuf = new ByteBuffer({size: this.packageDataToalLength})
                                for(let j = 0 ; j < data.packetNum; j++){
                                    let packageBuf =  this.packageDataArr[j];
                                    newBuf.put(packageBuf,0,packageBuf.length);
                                }
        
                                let copyBuf = newBuf.copy();
        
                                if(data.responsType==1){
                                    this.doSuccessMessage(copyBuf)
                                }
                                else if(data.responsType==3){
                                    this.doFailureMessage(copyBuf)
                                }
                                else{
                                    newDebug("未知的数据包类型:"+data.responsType);
                                }
        
                                this.packageDataToalLength = 0;
                                this.packageDataArr = null;
                            }
        
                        }
                    }
                }
            } 
        }
        
    }

    doSuccessMessage1(message){
        try {
            let html5RpcResultEvent = new Html5RpcResultEvent({"result":message});
            if(this.successFunc != null){
                this.successFunc.apply(null,[html5RpcResultEvent]);
                this.clean();
            }
        }
        catch(err){
            newDebug("收到数据 调用RPC成功序列化失败:"+err.message)
            let errorEvent = new Html5RpcFaultEvent({"code":Html5RpcFaultEvent.PARSE_ERROR,"reason":"序列化失败","detail":err.message});
            if(this.failureFunc != null){
                this.failureFunc.apply(null,[errorEvent]);
                this.clean();
            }
        }
    }

    doSuccessMessage(message){
        debug("doSuccessMessage 真实rpc返回值的长度为:"+message.length);
        if(isChenBinTest) {
            let html5RpcResultEvent = new Html5RpcResultEvent({"result":message});
            if(this.successFunc != null){
                this.successFunc.apply(null,[html5RpcResultEvent]);
                this.clean();
            }
        } else {
            try {
                let homeData = hessian.decode(message, '2.0');
                //debug("反编码成功")
                let html5RpcResultEvent = new Html5RpcResultEvent({"result":homeData});
                if(this.successFunc != null){
                    this.successFunc.apply(null,[html5RpcResultEvent]);
                    this.clean();
                }
            }
            catch(err){
                newDebug("收到数据 调用RPC成功序列化失败:"+err.message)
                let errorEvent = new Html5RpcFaultEvent({"code":Html5RpcFaultEvent.PARSE_ERROR,"reason":"序列化失败","detail":err.message});
                if(this.failureFunc != null){
                    this.failureFunc.apply(null,[errorEvent]);
                    this.clean();
                }
            }
        }
    }

    doFailureMessage(message){
        newDebug("收到数据 调用RPC失败序列化失败...");
        if(isChenBinTest) {
            let html5RpcFaultEvent = new Html5RpcFaultEvent({"result":message});
            if(this.failureFunc != null){
                this.failureFunc.apply(null,[html5RpcFaultEvent]);
                this.clean();
            }
        } else {
            try {
                let errorData = hessian.decode(message, '2.0');
                let html5RpcFaultEvent = new Html5RpcFaultEvent({"code":Html5RpcFaultEvent.SERVER_ERROR,"reason":errorData,"detail":errorData});
                if(this.failureFunc != null){
                    this.failureFunc.apply(null,[html5RpcFaultEvent]);
                    this.clean();
                }
            }
            catch(err){
                newDebug("收到数据 调用RPC失败序列化失败")
                let errorEvent = new Html5RpcFaultEvent({"code":Html5RpcFaultEvent.PARSE_ERROR,"reason":"序列化失败","detail":err.message});
                if(this.failureFunc != null){
                    this.failureFunc.apply(null,[errorEvent]);
                    this.clean();
                }
            }
        }
    }

    // beginSendSimple(operaion) {
    //     newDebug("向后台发送请求...");
    //     try{
    //         newDebug("发送的数据为:"+operaion.args);
    //         let buf = hessian.encode(operaion.args, '2.0');
    //         operaion.socket.send(buf);
    //     } catch(err){
    //         newDebug("RPC 发送简单数据序列化失败");
    //         let errorEvent = new Html5RpcFaultEvent({"code":Html5RpcFaultEvent.PARSE_ERROR,"reason":"序列化失败","detail":err.message});
    //         if(this.failureFunc != null){
    //             this.failureFunc.apply(null,[errorEvent]);
    //         }
    //     }
    // }

    beginSend(operaion){
        newDebug("向后台发送请求,pkgName="+operaion.pkgName+",methodName="+operaion.methordName+",uid="+operaion.uid);
        if(isChenBinTest){
            let requestInfoModel = {
                $class: 'com.springboot.websocket.demo.websocket.nopkmodel.RequestInfoModel',
                $:{
                    uuid:operaion.uid,
                    serviceName:operaion.pkgName,
                    methodName:operaion.methordName,
                    token:operaion.token,
                    parameters:operaion.args
                }
            }

            try{
                let uid = operaion.uid;
                newDebug("发送的数据为uid为:"+uid);
                let buf = hessian.encode(requestInfoModel, '2.0');
                newDebug("发送的二进制长度为:"+buf.length)
                operaion.socket.send(buf);
            }
            catch(err){
                newDebug("RPC 发送数据序列化失败")
                let errorEvent = new Html5RpcFaultEvent({"code":Html5RpcFaultEvent.PARSE_ERROR,"reason":"序列化失败","detail":err.message});
                if(this.failureFunc != null){
                    this.failureFunc.apply(null,[errorEvent]);
                }
            }
        }
        else{
            let requestInfoModel = {
                $class: 'com.springboot.websocket.demo.websocket.model.RequestInfoModel',
                $:{
                    serviceName:operaion.pkgName,
                    methodName:operaion.methordName,
                    parameters:operaion.args,
                    token:operaion.token
                }
            }
    
    
            let buf;
            try{
                let uid = operaion.uid;
                buf = hessian.encode(requestInfoModel, '2.0');
                newDebug("二进制分包的基块大小为:"+perPackageSize);
    
                newDebug("发送的数据为uid为:"+uid);
                newDebug("参数的二进制长度为:"+buf.length);
                
                if(buf.length > perPackageSize ) { //分包
    
                    let isJustOk = buf.length%perPackageSize == 0;
                    let packetNum = 0;
                    if(isJustOk == true){
                        packetNum = buf.length/perPackageSize;
                    }
                    else{
                        packetNum = Math.floor(buf.length/perPackageSize)+1;
                    }
                    newDebug("分包个数为:"+packetNum);
                    for(let packetIndex = 0 ; packetIndex < packetNum; packetIndex++){
                        let packageBuf = buf.slice(packetIndex*perPackageSize,(packetIndex+1)*perPackageSize);
                        let packageData = {
                            $class: 'com.springboot.websocket.demo.websocket.model.RequestPacketModel',
                            $:{
                                uuid:uid,
                                packetIndex:(packetIndex+1),
                                packetNum:packetNum,
                                requestInfoBytes:packageBuf
                            }
                        }
                        newDebug("当前分包索引为:"+(packetIndex+1)+',一共有包'+packetNum);
    
                        let packageGotohomeData = hessian.encode(packageData, '2.0');
                        newDebug("发送的二进制长度为:"+packageGotohomeData.length)
                        operaion.socket.send(packageGotohomeData);
                    }
    
                }
                else{//不分包
    
                    let oneData = {
                        $class: 'com.springboot.websocket.demo.websocket.model.RequestPacketModel',
                        $:{
                            uuid:uid,
                            packetIndex:1,
                            packetNum:1,
                            requestInfoBytes:buf
                        }
                    }
                    newDebug("发送的数据为uid为:"+uid)
                    newDebug("分包个数为:"+oneData.$.packetNum);
                    let gotohomeData = hessian.encode(oneData, '2.0');
                    newDebug("发送的二进制长度为:"+gotohomeData.length)
                    operaion.socket.send(gotohomeData);
    
    
                    /* 测试
                    let  newBuf = new ByteBuffer({
                        size: gotohomeData.length
                    })
                    newBuf.put(gotohomeData,0,gotohomeData.length);
                    let dd = newBuf.copy();
                    var res = hessian.decode(dd, '2.0');
                    debug(res);
                    */
                }
            }
            catch(err){
                newDebug("RPC 发送数据序列化失败")
                let errorEvent = new Html5RpcFaultEvent({"code":Html5RpcFaultEvent.PARSE_ERROR,"reason":"序列化失败","detail":err.message});
                if(this.failureFunc != null){
                    this.failureFunc.apply(null,[errorEvent]);
                }
            }
        }
    }

    clean(){
        this.uid = null;
        this.pkgName = null;
        this.methordName = null;
        this.args = null;
        this.successFunc = null;
        this.failureFunc = null;
        this.socket.removeOnMessageCallBack(this.onSocketMessage,this);
        this.socket = null;
        this.token = null;
    }
}

module.exports  = Html5RpcOperation;

