'use strict';
/**
 * Created by gongbin on 2017/6/22.
 */

const socketManager = require("./WebSocketManager");
const Html5RpcOperation = require("./Html5RpcOperation");
const uuidUtil = require('node-uuid');

const sendRpcInterval = require("./Html5RpcManager").sendRpcInterval;
const ISDEBUG = require("./Html5RpcManager").debug;
const debug = require('debug')('gongbin:Html5RpcService');


let newDebug = function(msg){
    console.log(msg);
    if(ISDEBUG){
        debug(msg);
    }
}


 class Html5RpcService{

    constructor(serviceUrl,connectSuccessCallBack=null,connectFailureCallBack=null){
        this.operationMap = new Map();
        this.url = serviceUrl;
        this.socket = socketManager.getWebSocket(serviceUrl);
        
        if(connectSuccessCallBack != null && connectSuccessCallBack != undefined){
            this.socket.on("socketConnectOpen",connectSuccessCallBack)
        }
        if(connectFailureCallBack != null && connectFailureCallBack != undefined){
            this.socket.on("socketConnectClose",connectFailureCallBack)
        }
        
        newDebug("准备与:"+serviceUrl+" 建立webscoket连接")
    }

    /**
     * Send a data message.
     *
     * @param {String} 包名
     * @param {String} 方法名
     * @param {Array} 参数
     * @public
     */
    send(pkgName,methordName,args,successFunc,failureFunc,token=null){
        let mySocket = this.socket;
        let myOperationMap = this.operationMap;

        let invokeFunc = function(pkgName,methordName,args,successFunc,failureFunc,token=null){
            let operationUid =  uuidUtil.v1();
          //  debug("发送请求,uuid="+operationUid+",pkgName="+pkgName+",methordName="+methordName)
            let operation = new Html5RpcOperation(operationUid,token,pkgName,methordName,args,mySocket,successFunc,failureFunc);
            myOperationMap.set(operationUid,operation);
            operation.beginSend(operation);
        }

        if(this.socket != null && this.socket.readyState == 1){ //1 是OPEN状态
            invokeFunc(pkgName,methordName,args,successFunc,failureFunc,token);
        }
        else{ //这里应该循环检查是否连上，连上再发
            newDebug("websocket 处在不通的状态,url="+this.socket.url)
            newDebug("开始循环检测websocket的状态,当为1的时候表示连通了")
          
            let oneSecond = sendRpcInterval; // one second = 1000 x 1 ms

            let timeoutFlag = setTimeout(function() {
                newDebug("websocket 状态为:"+mySocket.readyState+",url="+mySocket.url)
            
                if(mySocket != null && mySocket.readyState == 1){
                    clearTimeout(timeoutFlag);
                    invokeFunc(pkgName,methordName,args,successFunc,failureFunc,token);
                }
            }, oneSecond);
        }
    }

}
module.exports  = Html5RpcService;

