'use strict';
/**
 * Created by gongbin on 2017/6/22.
 */
const hessian = require('hessian.js');
const EventEmitter = require("events").EventEmitter;

const debug = require('debug')('gongbin:WebSocketManager');
const Html5TopicManager = require("./Html5TopicManager")
const ISDEBUG = require("./Html5RpcManager").debug;
const socketReconnectInterval = require("./Html5RpcManager").socketReconnectInterval;
const isReconnectSocket = require("./Html5RpcManager").isReconnectSocket;
const runtimeName = require("./Html5RpcManager").runtimeName;
const isChenBinTest = require("./Html5RpcManager").isChenBinTest;

let newDebug = function(msg){
    console.log(msg);
    if(ISDEBUG){
        debug(msg);
    }
}

 class WebSocketManager{



    constructor(){
        this.socketMap = new Map();
    }

    getWebSocket(socketUrl){
        if(this.socketMap.has(socketUrl)){
            return this.socketMap.get(socketUrl);
        }
        else{
            let newMySocket = this.createNewSocket(socketUrl);
            return newMySocket;
        }
    }

    createNewSocket(socketUrl){
        let newSocket = null;
        if(runtimeName == "native"){
            let WebSocket = require('ws'); 
            newSocket = new WebSocket(socketUrl);
        }
        else{
            newSocket = new WebSocket(socketUrl);
            newSocket.binaryType = "arraybuffer";
        }

        
        let newMySocket = new MySocket(newSocket);
        this.socketMap.set(socketUrl,newMySocket);
        return newMySocket;
    }


    deleteSocket(socketUrl){
        if(this.socketMap.has(socketUrl)){
            let oldSocket = this.socketMap.get(socketUrl);
            oldSocket.deleteSocket();
            this.socketMap.delete(socketUrl);
        }
    }


    reConnect(oldSocket){
        newDebug(socketReconnectInterval+"毫秒后 socket开始重连:"+oldSocket.url)
       

            setTimeout(function() {
                 let newSocket = new WebSocket(oldSocket.url);
                 webSocketManager.getWebSocket(oldSocket.url).setNewWebSocket(newSocket);
            }, socketReconnectInterval);

       
    }


}



class MySocket extends EventEmitter{
    
    constructor(webSocket){
        super();
        this.webSocket = webSocket;
        this.webSocket.onmessage = this.onSocketMessage;
        this.webSocket.onopen = this.onSocketOpen;
        this.webSocket.onclose = this.onSocketClose;
        this.webSocket.onerror = this.onSocketError;
        this.messageCallBacks = new Set();
      //  debug("MySocket构造函数")
    }

    setNewWebSocket(newWebSokct){
        this.webSocket.onmessage = null;
        this.webSocket.onopen = null;
        this.webSocket.onclose = null;
        this.webSocket.onerror = null;

        this.webSocket = newWebSokct;
        this.webSocket.onmessage = this.onSocketMessage;
        this.webSocket.onopen = this.onSocketOpen;
        this.webSocket.onclose = this.onSocketClose;
        this.webSocket.onerror = this.onSocketError;
    }

    addOnMessageCallBack(callBack,operation){
        this.messageCallBacks.add({"callBack":callBack,"op":operation});
    }

    removeOnMessageCallBack(callBack,operation){
        for(let item of this.messageCallBacks.values()){
            if(item.callBack == callBack && item.op == operation){
                this.messageCallBacks.delete(item);
            }
        }

    }

    get className(){
        return "MySocket";
    }

    get url(){
        return this.webSocket.url;
    }

    get readyState(){
        return this.webSocket.readyState;
    }

    send(data){
        this.webSocket.send(data);
    }

    onSocketOpen(e){
        let mySocket = webSocketManager.getWebSocket(this.url)
        mySocket.emit("socketConnectOpen",this.url)
        newDebug("socket连接成功:"+mySocket.className)
    }

    onSocketMessage(event){
        try{
            let responsePacketModel = null;
            if(isChenBinTest){
                // responsePacketModel  = hessian.decode(event.data, '2.0');
                // let mySocket = webSocketManager.getWebSocket(this.url);
                // for(let item of mySocket.messageCallBacks.values()){
                //     item.callBack.apply(item.op,[responsePacketModel]);
                // }
                let responseInfoModel = null;
                if(runtimeName == "brower"){
                    responseInfoModel  = hessian.decode(new Buffer(event.data), '2.0');
                }
                else{
                    responseInfoModel  = hessian.decode(event.data, '2.0');
                }
                if(responseInfoModel.responsType == 2){
                    newDebug("收到推送消息");
                    Html5TopicManager.getInstance().onTopicMessage(responseInfoModel);
                } else {
                    newDebug("收到RPC消息");
                    let mySocket = webSocketManager.getWebSocket(this.url)
                    for(let item of mySocket.messageCallBacks.values()){
                        item.callBack.apply(item.op,[responseInfoModel]);
                    }
                }
            }
            else{
                if(runtimeName == "brower"){
                    responsePacketModel  = hessian.decode(new Buffer(event.data), '2.0');
                }
                else{
                    responsePacketModel  = hessian.decode(event.data, '2.0');
                }

                if(responsePacketModel.responsType == 2){
                    newDebug("收到推送消息")
                }
                else{
                    newDebug("收到RPC消息")
                }
    
    
                if(responsePacketModel.responsType == 2){ //消息
                    Html5TopicManager.getInstance().onTopicMessage(responsePacketModel);
                }
                else{
    
                    let mySocket = webSocketManager.getWebSocket(this.url)
                    for(let item of mySocket.messageCallBacks.values()){
                        item.callBack.apply(item.op,[responsePacketModel]);
                    }
                }
            }
 
        }
        catch(err){
            newDebug("反序列化失败:"+err.message);
        }

    }

    onSocketClose({code=1,reason="close",wasClean=true}){
        let mySocket = webSocketManager.getWebSocket(this.url)
        mySocket.emit("socketConnectClose",this);
        newDebug("socket关闭:"+mySocket.url+',原因为:'+reason)
       
        //需要重连
        if(isReconnectSocket == true){
             webSocketManager.reConnect(this);
        }
       

    }

    onSocketError(e){
        let mySocket = webSocketManager.getWebSocket(this.url)
        mySocket.emit("socketConnectError",this);
        newDebug("socket出错:"+e.message+", "+mySocket.url)
    }

    deleteSocket(){
        this.webSocket.onopen = null;
        this.webSocket.onMessage = null;
        this.webSocket.onclose = null;
        this.webSocket.onerror = null;
        this.messageCallBacks = null;
        this.webSocket = null;
    }


}

const webSocketManager = new WebSocketManager();
module.exports  = webSocketManager;